// Protocol Buffers - Google's data interchange format
// Copyright 2023 Google Inc.  All rights reserved.
//
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file or at
// https://developers.google.com/open-source/licenses/bsd

edition = "2023";

package protobuf_editions_test.edition2023;

option java_package = "com.google.protobuf.editions.test.edition2023";
option java_outer_classname = "Edition2023NoMultipleFilesName";

message Edition2023JavaNoMultipleFilesMessage {
  int32 foo = 1;

  message NestedMessage {
    int32 bar = 1;
  }

  enum NestedEnum {
    NESTED_ENUM_UNKNOWN = 0;
  }
}

enum Edition2023JavaNoMultipleFilesEnum {
  JAVA_NO_MULTIPLE_FILES_ENUM_UNKNOWN = 0;
}

service Edition2023JavaNoMultipleFilesService {
  // Edition2023JavaNoMultipleFilesService.Method
  rpc Method(Edition2023JavaNoMultipleFilesMessage.NestedMessage)
      returns (Edition2023JavaNoMultipleFilesMessage) {}
}
