// Protocol Buffers - Google's data interchange format
// Copyright 2025 Google LLC.  All rights reserved.
//
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file or at
// https://developers.google.com/open-source/licenses/bsd

#include "google/protobuf/os_macros_restore.inc"

// For legacy reasons, the major/minor macros are not handled in the include
// above.
#pragma pop_macro("major")
#pragma pop_macro("minor")
