// Protocol Buffers - Google's data interchange format
// Copyright 2023 Google Inc.  All rights reserved.
//
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file or at
// https://developers.google.com/open-source/licenses/bsd

edition = "2023";

package protobuf_editions_test.edition2023;

option go_api_flag = "OPEN_TO_OPAQUE_HYBRID";

message Edition2023GoApiFlagNone {
  int32 foo = 1;
}

message Edition2023GoApiFlagOpen {
  option go_api_flag = "OPEN_V1";

  int32 foo = 1;
}

message Edition2023GoApiFlagHybrid {
  option go_api_flag = "OPEN_TO_OPAQUE_HYBRID";

  int32 foo = 1;
}

message Edition2023GoApiFlagOpaque {
  option go_api_flag = "OPAQUE_V0";

  int32 foo = 1;
}
