// Protocol Buffers - Google's data interchange format
// Copyright 2023 Google Inc.  All rights reserved.
//
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file or at
// https://developers.google.com/open-source/licenses/bsd

edition = "2023";

package protobuf_editions_test.edition2023;

import "third_party/golang/protobuf/v2/src/google/protobuf/go_features.proto";
import "google/protobuf/cpp_features.proto";

option features.(pb.go).api_level = API_OPAQUE;

message Edition2023ImportFeatureMessage {
  string view = 1 [features.(pb.cpp).string_type = VIEW];
}
