<?php
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: google/protobuf/descriptor.proto

namespace Google\Protobuf\Internal\FeatureSet\VisibilityFeature;

use UnexpectedValueException;

/**
 * Protobuf type <code>google.protobuf.FeatureSet.VisibilityFeature.DefaultSymbolVisibility</code>
 */
class DefaultSymbolVisibility
{
    /**
     * Generated from protobuf enum <code>DEFAULT_SYMBOL_VISIBILITY_UNKNOWN = 0;</code>
     */
    const DEFAULT_SYMBOL_VISIBILITY_UNKNOWN = 0;
    /**
     * Default pre-EDITION_2024, all UNSET visibility are export.
     *
     * Generated from protobuf enum <code>EXPORT_ALL = 1;</code>
     */
    const EXPORT_ALL = 1;
    /**
     * All top-level symbols default to export, nested default to local.
     *
     * Generated from protobuf enum <code>EXPORT_TOP_LEVEL = 2;</code>
     */
    const EXPORT_TOP_LEVEL = 2;
    /**
     * All symbols default to local.
     *
     * Generated from protobuf enum <code>LOCAL_ALL = 3;</code>
     */
    const LOCAL_ALL = 3;
    /**
     * All symbols local by default. Nested types cannot be exported.
     * With special case caveat for message { enum {} reserved 1 to max; }
     * This is the recommended setting for new protos.
     *
     * Generated from protobuf enum <code>STRICT = 4;</code>
     */
    const STRICT = 4;

    private static $valueToName = [
        self::DEFAULT_SYMBOL_VISIBILITY_UNKNOWN => 'DEFAULT_SYMBOL_VISIBILITY_UNKNOWN',
        self::EXPORT_ALL => 'EXPORT_ALL',
        self::EXPORT_TOP_LEVEL => 'EXPORT_TOP_LEVEL',
        self::LOCAL_ALL => 'LOCAL_ALL',
        self::STRICT => 'STRICT',
    ];

    public static function name($value)
    {
        if (!isset(self::$valueToName[$value])) {
            throw new UnexpectedValueException(sprintf(
                    'Enum %s has no name defined for value %s', __CLASS__, $value));
        }
        return self::$valueToName[$value];
    }


    public static function value($name)
    {
        $const = __CLASS__ . '::' . strtoupper($name);
        if (!defined($const)) {
            throw new UnexpectedValueException(sprintf(
                    'Enum %s has no value defined for name %s', __CLASS__, $name));
        }
        return constant($const);
    }
}

