// Protocol Buffers - Google's data interchange format
// Copyright 2024 Google Inc.  All rights reserved.
//
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file or at
// https://developers.google.com/open-source/licenses/bsd

edition = "2023";

package google.protobuf.python.internal;

message SelfRecursive {
  SelfRecursive sub = 1;
  int32 i = 2;
  SelfRecursive sub_group = 3 [features.message_encoding = DELIMITED];
}

message IndirectRecursive {
  IntermediateRecursive intermediate = 1;
}

message IntermediateRecursive {
  IndirectRecursive indirect = 1;
}
