// Protocol Buffers - Google's data interchange format
// Copyright 2016 Google Inc.  All rights reserved.
//
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file or at
// https://developers.google.com/open-source/licenses/bsd

syntax = "proto2";

package objc.protobuf.tests.chain;

import "objectivec/Tests/unittest.proto";

option objc_class_prefix = "Chain";

// The Root for this file should end up adding the local extension and merging
// in the extensions from unittest.proto's Root.

message CMessage {
  optional int32 my_field = 1;
}

extend objc.protobuf.tests.TestAllExtensions {
  optional int32 chain_c_extension = 10003;
}
