// Protocol Buffers - Google's data interchange format
// Copyright 2026 Google LLC.  All rights reserved.
//
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file or at
// https://developers.google.com/open-source/licenses/bsd

syntax = "proto2";

package protobuf.rust.test.same_name_direct_deps;

option java_package = "protobuf.rust.test.same_name_direct_deps";
option java_multiple_files = true;
option java_outer_classname = "SameNameDirectDeps";

import "rust/test/p/parent.proto";
import "rust/test/q/parent.proto";

message Child {
  optional p.Parent p = 1;
  optional q.Parent q = 2;
}
