// Protocol Buffers - Google's data interchange format
// Copyright 2023 Google Inc.  All rights reserved.
//
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file or at
// https://developers.google.com/open-source/licenses/bsd

edition = "2023";

package protobuf_editions_test.edition2023;

import "editions/codegen_tests/proto2_optional.proto";

message Edition2023ImportMessage {
  protobuf_editions_test.google.protobuf.Proto2Optional sub_message_field = 1;
}
