// Protocol Buffers - Google's data interchange format
// Copyright 2023 Google LLC.  All rights reserved.
//
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file or at
// https://developers.google.com/open-source/licenses/bsd

edition = "2023";

option features.enum_type = CLOSED;
// option java_multiple_files = true;

enum EnumWithNoPackage {
  CELSIUS = 1;
  FAHRENHEIT = 2;
}

message MessageWithEnumUpbTest {
  enum EnumWithNoPackageInMessage {
    UNKNOWN = 0;
    AB_1 = 1;
    CD_2 = 2;
    EF_3 = 3;
    GH_4 = 4;
  }
}
