// Protocol Buffers - Google's data interchange format
// Copyright 2008 Google Inc.  All rights reserved.
//
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file or at
// https://developers.google.com/open-source/licenses/bsd

syntax = "proto2";

package protobuf.kotlin.test;

option java_package = "com.google.protobuf.kotlin.test";
option java_multiple_files = true;

message ExampleExtensibleMessage {
  extensions 10 to 20;
}

extend ExampleExtensibleMessage {
  repeated int32 repeated_extension = 10;
  repeated int32 different_extension = 11;
  optional int32 int32_extension = 12;
}
