syntax = "proto2";

package google.protobuf;

option java_package = "com.google.protobuf";
option java_multiple_files = true;

option java_generic_services = true;

message MultipleFilesProto2Message {
  optional int32 field = 1;
}

enum MultipleFilesProto2Enum {
  MULTIPLE_FILES_PROTO2_ENUM_UNKNOWN = 0;
}

service MultipleFilesProto2Service {
  // Test method.
  rpc Foo(MultipleFilesProto2Message) returns (MultipleFilesProto2Message) {}
}
