package com.google.protobuf.kotlin

@Target(AnnotationTarget.CLASS, AnnotationTarget.FUNCTION)
@Retention(AnnotationRetention.BINARY)
@OptIn(kotlin.experimental.ExperimentalObjCRefinement::class)
@kotlin.native.HidesFromObjC
@OnlyForUseByGeneratedProtoCode
annotation class ProtoHiddenFromObjC
