edition = "2023";

package google.protobuf;

option java_package = "com.google.protobuf";

message EmptyMessage {}

service GeneratorNamesConflictingService {
  // Test method.
  rpc Foo(EmptyMessage) returns (EmptyMessage) {}
}
