// This file contains messages that are used to test the field_mask_util.
// It has no other purpose beyond that.

edition = "2023";

package third_party.protobuf.util.python.testdata;

import "google/protobuf/timestamp.proto";

option java_multiple_files = true;

message TimestampWrapper {
  google.protobuf.Timestamp timestamp = 1;
}

message MapWrapper {
  map<string, string> map = 1;
}
