// Protocol Buffers - Google's data interchange format
// Copyright 2008 Google Inc.  All rights reserved.
//
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file or at
// https://developers.google.com/open-source/licenses/bsd

// Author: Darick Tong (darick@google.com)
//
// A proto file with extensions.

// LINT: LEGACY_NAMES

edition = "2023";

package proto2_unittest;

option features.utf8_validation = NONE;

message MessageToBeExtended {
  extensions 1 to max;
}

message MyNonNestedExtension {}

extend MessageToBeExtended {
  MyNonNestedExtension nonNestedExtension = 1;
  string if = 2000;
}
