// Protocol Buffers - Google's data interchange format
// Copyright 2008 Google Inc.  All rights reserved.
//
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file or at
// https://developers.google.com/open-source/licenses/bsd

syntax = "proto3";

package protobuf.kotlin.generator;

option java_package = "com.google.protobuf.kotlin.generator";
option java_multiple_files = true;

enum NestedEnum {
  FOO = 0;
}

message MultipleFilesMessageA {}

message MultipleFilesMessageB {}
