// Protocol Buffers - Google's data interchange format
// Copyright 2008 Google Inc.  All rights reserved.
//
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file or at
// https://developers.google.com/open-source/licenses/bsd

// Author: Darick Tong (darick@google.com)
//
// A proto file with extensions for a MessageLite messages.

// LINT: LEGACY_NAMES

edition = "2023";

package proto2_unittest;

option optimize_for = LITE_RUNTIME;

message MessageLiteToBeExtended {
  extensions 1 to max;
}

message MyNonNestedExtensionLite {}

extend MessageLiteToBeExtended {
  MyNonNestedExtensionLite nonNestedExtensionLite = 1;
  bool package = 2006;
}
