edition = "2023";

package srcsless_library_test;

// The purpose of this proto is for a test where a proto_library target that has
// no srcs (an "alias library") can be used in the middle between this parent
// target and this imported file.
import "rust/test/srcsless_library_test_child.proto";

option java_multiple_files = true;

message Parent {
  Child child = 1;
}
