// Protocol Buffers - Google's data interchange format
// Copyright 2023 Google Inc.  All rights reserved.
//
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file or at
// https://developers.google.com/open-source/licenses/bsd

edition = "2023";

package protobuf_editions_test.edition2023;

message Edition2023CTypeMessage {
  string str = 1;
  string cord = 2 [ctype = CORD];

  bytes str_bytes = 4;
  bytes cord_bytes = 5 [ctype = CORD];

  extensions 100 to 199;
}

message Edition2023CTypeMessageExt {
  extend Edition2023CTypeMessage {
    string str_extension = 100;
    bytes str_bytes_extension = 102;
  }
}
