edition = "2023";

// LINT: LEGACY_NAMES

package protobuf_editions_test.edition2023.BadPackageName;

message bad_message_name {
  string BadFieldName = 1;
  string BAD_FIELD_NAME = 2;
  string badFieldName = 3;
}

enum bad_enum_name {
  BAD_ENUM_VALUE_0 = 0;
  bad_enum_value = 1;
  badEnumValue = 2;
}
