// Protocol Buffers - Google's data interchange format
// Copyright 2024 Google LLC.  All rights reserved.
//
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file or at
// https://developers.google.com/open-source/licenses/bsd

syntax = "proto2";

package imported_types;

message ImportedMessage {
  optional int32 num = 1;
}

enum ImportedEnum {
  IMPORTED_ENUM_UNKNOWN = 0;
  IMPORTED_ENUM_BAR = 1;
}
